using System.Text.Json;
using WF_WebAdmin.Model;

namespace WF_WebAdmin.Service;

public class QuizServiceStub: IQuizService
{
    private readonly string _jsonFilePath = Path.Combine(Environment.CurrentDirectory, "wwwroot", "fake_data_quiz.json");

    public async Task saveQuizJson(List<Quiz> quizzes)
    {
        var json = JsonSerializer.Serialize(quizzes, new JsonSerializerOptions { WriteIndented = true });
        await File.WriteAllTextAsync(_jsonFilePath, json);
    }

    public async Task addQuiz(Quiz quiz)
    {
        var data = await getQuizzes();
        quiz.Id = data.Count > 0 ? data.Max(p => p.Id) + 1 : 1;
        data.Add(quiz);
        await saveQuizJson(data);
    }

    public async Task updateQuiz(Quiz quiz)
    {
        var data = await getQuizzes();
        var existingQuiz = data.FirstOrDefault(q => q.Id == quiz.Id);
        if (existingQuiz != null)
        {
            existingQuiz.Question = quiz.Question;
            existingQuiz.AnswerA = quiz.AnswerA;
            existingQuiz.AnswerB = quiz.AnswerB;
            existingQuiz.AnswerC = quiz.AnswerC;
            existingQuiz.AnswerD = quiz.AnswerD;
            existingQuiz.CAnswer = quiz.CAnswer;
            existingQuiz.IsValid = quiz.IsValid;
            existingQuiz.UserProposition = quiz.UserProposition;
            await saveQuizJson(data);
        }
    }

    public async Task removeQuiz(int id)
    {
        var data = await getQuizzes();
        var quiz = data.FirstOrDefault(q => q.Id == id);
        if (quiz != null)
        {
            data.Remove(quiz);
            await saveQuizJson(data);
        }
    }

    public Task validateQuiz(int id)
    {
        throw new NotImplementedException();
    }

    public async Task<List<Quiz>> getQuizzes()
    {
        if (!File.Exists(_jsonFilePath))
        {
            Console.Out.WriteLine($"{_jsonFilePath} not found");
            return new List<Quiz>();
        }

        var json = await File.ReadAllTextAsync(_jsonFilePath);
        return JsonSerializer.Deserialize<List<Quiz>>(json) ?? new List<Quiz>();
    }

    public async Task<List<Quiz>> getQuizzesToValidate()
    {
        var quizzes = await getQuizzes();
        return quizzes.Where(quiz => quiz.IsValid == false).ToList();
    }

    public async Task<Quiz> getQuiz(int id)
    {
        var data = await getQuizzes();
        var q = data.FirstOrDefault(p => p.Id == id);
        if (q != null)
        {
            return q;
        }
        return null;
    }

    public async Task<List<Quiz>> getSommeQuiz(int nb, int page)
    {
        var data = await getQuizzes();
        if ((page - 1) * nb + nb > data.Count())
        {
            if(nb > data.Count())
            {
                return data.GetRange(0, data.Count()-1);
            }
            return data.GetRange(data.Count() - nb, nb);
        }
        return data.GetRange((page - 1) * nb, nb);
    }

    public async Task<int> getNbQuiz()
    {
        var data = await getQuizzes();
        return data.Count;
    }
}
